/*
 *  knock.c
 *
 *  Copyright (c) 2004 by Beretta Andrea <beretta.andrea@libero.it>
 *  and Bianchi Riccardo <ikir@iol.it>
 *
 *  Thanks to Marco Balduzzi <embyte@spine-group.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *
 */

#include "knock.h"


/***********************
* Main function
************************/
int main(int argc, char** argv)
{	
	/*command line option*/
	int option ;
	char *conf_file=NULL;
	char *seq_name=NULL;
	char *dest_ip=NULL;
	char *iface_name=NULL;
	/*string ip representation*/
	char *Ip_String=NULL;
	/*char * used with strsep in encryption of ip address*/
	char *tmp_ip_string;
	char *ip_bit;
	/* error buffer */
	char err_buf[LIBNET_ERRBUF_SIZE];	
	/*pointer used to scan knock list*/	
	PMList* lp;	
	/* sequence that will be crypted */
	u_char *sequence;
	u_char byte;
	/*flag & index*/
	int seq_found=0;
	int index;
	int flag_write=1;

	/*only root can run the program*/
	if( getuid() )
		{
		printf("You should be root to run this program\n");
		exit(EXIT_FAILURE);
		}

	/*parse the command line options*/
	while ((option = getopt (argc, argv, "f:s:d:i:g:h"))!=-1)
		{//</while options>
		switch(option) 
			{//<switch>
			case 'd':
				dest_ip=malloc(strlen(optarg)+1);
				strcpy(dest_ip, optarg);
				break;
			case 'f': 
				conf_file=malloc(strlen(optarg)+1);
				strcpy(conf_file, optarg); 
				break;
			case 'g':
				Ip_String=malloc(strlen(optarg)+1); 
				strcpy(Ip_String, optarg);
				break;				
			case 'i':
				iface_name=malloc(strlen(optarg)+1);
				strcpy(iface_name, optarg);
				break;
			case 's':
				seq_name=malloc(strlen(optarg)+1);
				strcpy(seq_name, optarg);  
				break;
			case '?':
				printf("Unknown option -%c\n", optopt);
			case 'h': /* fallthrough */
			default: 
				help();
				exit(EXIT_FAILURE);
			}//</switch>
		}//</while options>
		
	/*check some values*/
	if (seq_name==NULL || dest_ip==NULL || iface_name==NULL)
		{
		help();
		exit(EXIT_FAILURE);
		}
			
	/*if not specified we use standard conf_file*/	
	if(conf_file==NULL)
		{//<conf file>
		char *homedir=NULL;
		homedir=getenv("HOME");
		if (!homedir)
			{
			printf("ERROR : Can't get $HOME value\n");		
			exit(EXIT_FAILURE);
			}
		conf_file=malloc(strlen(homedir)+10);	
		strcpy(conf_file, homedir);
		strcat(conf_file, "/.knockrc");
		}//</conf_file>
	
	/*parse the config file*/
	parse_config(conf_file);
	
	/*check if exist the sequence seq_name*/
	for (lp=knocks_list; lp; lp=lp->next) 
		{//<for>
		knock=(knock_t*)lp->data;
		if (!strncmp(knock->name,seq_name,sizeof( knock->name)) )
			{//<seq_name exist>
			/*set the flags for the tcp packets*/			
			if(knock->flag_fin == 1)
				flags += TH_FIN;
			if(knock->flag_syn == 1)
				flags += TH_SYN;		 
			if(knock->flag_rst == 1)
				flags += TH_RST;		 
			if(knock->flag_psh == 1)
				flags += TH_PUSH;		 		 
			if(knock->flag_ack == 1)
				flags += TH_ACK;		 		 
			if(knock->flag_urg == 1)
				flags += TH_URG;		 		 			
			seq_found=1;
			break;
			}//</seq_name exist> 			
		}//</for>
	
	/*the sequence wasn't found*/
	if(!seq_found)
		{
		printf("The sequence named %s doesn't exist\n", seq_name);
		exit(EXIT_FAILURE);
		}

	/*init libnet*/
	if ((lnet=libnet_init(LIBNET_RAW4, iface_name, err_buf))==NULL)
		{
		printf ("Libnet Error: %s\n", err_buf);
		exit(EXIT_FAILURE);
		}
     	
	/* detect iface ip and check */
	if ((IfaceIp = libnet_get_ipaddr4(lnet))==-1)
		{
		printf ("Libnet Error: %s\n", libnet_geterror(lnet));
		exit(EXIT_FAILURE);
		}
	 
	/*resolve knockserv name in ip address*/
	if ((KnockServIp = libnet_name2addr4(lnet, dest_ip, LIBNET_RESOLVE))==-1)
		{
		printf ("Libnet Error: %s\n", libnet_geterror(lnet));
		exit(EXIT_FAILURE);
		}
		
	/*convert iface address to text representation*/
	if (! Ip_String ) 
		{
		Ip_String=malloc(20); 
		inet_ntop (AF_INET, &IfaceIp, Ip_String, 20);
		#ifdef DEBUG
			printf("Interface ip : %s\n" , Ip_String); 
		#endif
		}
		
	/*seed to produce a different pseudo-random series each time program is run,*/
	srand (time (0));

	/*generate the smokescreen*/
	gen_smoke();

	/* sequence that will be crypted */
	sequence=(u_char *) calloc (knock->seqcount, sizeof(u_char));
	strncpy (sequence, knock->sequence, knock->seqcount);

	/*init mcrypt*/	
	mcrypt_init();
		
	/*build the packets*/
	printf("\n** Start writing packets on the wire **\n");
	while(flag_write)
		{//<while flag>	
		for (index=0; index < knock->seqcount; )
			{//<for>
			randomize();
			#ifdef MODULE_KNOCK
				sleep(1);
			#endif
			/*first part of the knock*/		
			switch(knock->protocol[index])
				{//<switch>					
				case IPPROTO_UDP:
					/*encrypt port*/
					mcrypt_generic (hd, &sequence[index], 1);
					/*build udp packet*/					
					build_udp(map((u_short)sequence[index]));
					#ifdef VERBOSE
						printf("%s",WHITE);
						printf("[UDP] %d\t\tKnock packet\n", map((u_short)sequence[index]));
						printf("%s",DEFAULT);
					#endif					
					break;
				case IPPROTO_TCP: 
					/*encrypt port*/
					mcrypt_generic (hd, &sequence[index], 1);
					/*build tcp packet*/
					build_tcp(map((u_short)sequence[index]));
					#ifdef VERBOSE
						printf("%s",WHITE);
						printf("[TCP] %d\t\tKnock packet\n", map((u_short)sequence[index]));
						printf("%s",DEFAULT);						
					#endif										
					break;
				case IPPROTO_ICMP:
					/*build icmp packet*/
					build_icmp();	
					#ifdef VERBOSE
						printf("%s[ICMP]\t\t\tKnock packet%s\n", WHITE, DEFAULT);
					#endif													
					break;
				}//</switch>
			index++;
			}//</for>
	
		/*build tcp packets for the bits of the ip address*/ 
		tmp_ip_string=Ip_String;
		while((ip_bit = strsep(&tmp_ip_string, ".")))
			{
			randomize();				
			/*encrypt byte*/
			byte = (u_char) atoi(ip_bit);
			mcrypt_generic (hd, &byte, 1);
			/*build tcp packet*/
			#ifdef MODULE_KNOCK
				sleep(1);
			#endif	
			build_tcp(map((u_short) byte));
			#ifdef VERBOSE
				printf("%s",WHITE);
				printf("[TCP] %d\t\tKnock packet\n", map((u_short)byte));
				printf("%s",DEFAULT);				
			#endif	
			}
		
		/*packet for the last bit of ip was sent*/	
		if(ip_bit==NULL)
			{
			randomize();	 
			flag_write=0;
			}
		}//</while flag>		
	
	printf("** Knock Sequence sent to %s **\n\n", dest_ip);																			

	/* deinit and close mcrypt */
	mcrypt_generic_deinit (hd);
  	mcrypt_module_close	(hd); 
  			
	/* destroy all */
	libnet_destroy(lnet);	
	exit(EXIT_SUCCESS);
}



/*****************************
* Parse configuration file
*****************************/
void parse_config(char *filename)
{
	/*flag used to parse seq*/
	int in_seq_flag=0; 		
	/*number of ports in a range of portspan*/
	int range_count=0;
	FILE *fp = NULL;
	char *line=malloc(PATH_MAX + 1);
	
	/*open the file*/
	if((fp = fopen(filename, "r")) == NULL)
		{
		printf("Cannot open config file %s\n", filename);
		exit(EXIT_FAILURE);
		}

	while(fgets(line, PATH_MAX, fp)) 
		{//<walking conf file>
		/*trim spaces and newline*/
		trim(line);
		/*ignore comment line*/
		if(!strlen(line) || *line=='#')
			continue;
		/*cypher to use with gcrypt*/	
		else if (strncasecmp(line, "cypher=",7)==0)
			{//<cypher>
			snprintf(CryptAlgName, 100, line+7);			
			#ifdef DEBUG	
				printf("\nThe cypher is \"%s\"\n", CryptAlgName);
			#endif
			continue;	
			}//</cypher>

		/*key used to crypt sequences*/
		else if (strncasecmp(line, "password=",9)==0)
			{//<key>
			snprintf(password, 256, line+9);					
			#ifdef DEBUG			
				printf("\nThe password is \"%s\"\n",password);
			#endif	
			continue;
			}//</key>	
			
		/*Inizialization Vector*/
		else if (strncasecmp(line, "iv=",3)==0)
			{//<key>
			snprintf(temp_iv, 16, line+3);
			#ifdef DEBUG			
				printf("\nThe IV is \"%s\"\n",temp_iv);
			#endif	
			continue;
			}//</key>		
			
		/*range of port used by the server*/
		else if (strncasecmp(line, "span=",5)==0)
			{//<portspan>
			char *tmp_line;
			char *temp;
			char *min;
			char *max;
			snprintf(line, strlen(line), line+5);
			tmp_line=line;
		
			#ifdef DEBUG	
				printf("\nThe port span is ");
			#endif		
			while((temp = strsep(&tmp_line, ","))) 
				{//<while>
				port_range = malloc(sizeof(portspan_t));
				if(port_range == NULL) 
					{
					printf("Error: can't alloc portspan structure\n");
					exit(EXIT_FAILURE);
					}
				min = strsep(&temp, "-");
			 	min =trim(min);
			 	port_range->minrange=atoi(min);
			 	max= strsep(&temp, "-");
			 	max =trim(max);	
			 	port_range->maxrange=atoi(max);
				range_count+=(port_range->maxrange - port_range->minrange);
			 	if (range_count>255)
			 		{
			 		printf("\nError in portspan : the number of ports must be lower than 256 \n");
			 		exit(EXIT_FAILURE);
			 		}	 	
				portspan_list=list_add(portspan_list, port_range);
				#ifdef DEBUG
					printf("%d) %d-%d\t",list_count(portspan_list) ,port_range->minrange,port_range->maxrange);	
				#endif	
				}//</while>
			#ifdef DEBUG
				printf("\n");
			#endif	
			continue;
			}//</portspan>
						
		/*we've found the begin of a new <seq> section*/
		else if (strncasecmp(line, "<seq>",5)==0)
			{//<found new seq>
			in_seq_flag=1;
			
			knock = malloc(sizeof(knock_t));
			if(knock == NULL) 
				{
				printf("Error: can't alloc knock structure\n");
				exit(EXIT_FAILURE);
				}
			continue;	
			}//</found new seq>
		
		/*parsing <seq> data */
		else if(in_seq_flag)
			{//<parse sequence>
			
			/*end of seq section*/
			if(strncasecmp(line, "</seq>",6)==0)
				{//<end sequence>		
				#ifdef DEBUG	
					int i;			
					printf("knock packets\n");
					for(i = 0; i < knock->seqcount; i++) 
						{//<for>
						switch(knock->protocol[i])
							{//<switch>
							case IPPROTO_UDP:
								printf("\t%d) %d : udp\n", i+1, knock->sequence[i]);
								break;
							case IPPROTO_TCP: 
								printf("\t%d) %d : tcp\n", i+1, knock->sequence[i]);
								break;
							case IPPROTO_ICMP:
								printf("\t%d) ICMP\n", i+1);
								break;
							}//</switch>
						}//</for>				
				#endif
				in_seq_flag=0;
				continue;
				}//</end sequence>
	
			/*name of the sequence*/ 	
			else if(strncasecmp(line, "name=",5)==0)
				{//<seq name>
				/*start fill knock struct*/
				snprintf(knock->name, sizeof(knock->name), line+5);		
				knock->seqcount = 0;
				knock->flag_fin = 0;		
				knock->flag_syn = 0;
				knock->flag_rst = 0;
				knock->flag_psh = 0;
				knock->flag_ack = 0;								
				knock->flag_urg = 0;		
				knocks_list = list_add(knocks_list, knock);
				#ifdef DEBUG
					printf("\nSequence %d) %s\n ",list_count(knocks_list), knock->name);
				#endif
				continue;
				}//</seq name>

 			/*parsing knock sequence*/	
			else if(strncasecmp(line, "sequence=",9)==0)
				{//<knock sequence>
				char *tmp;
				char *num;
				char *protocol;
				char *port;
		
				snprintf(line, strlen(line), line+9);
				tmp=line;
				while((num = strsep(&tmp, ","))) 
					{//<while>
					if(knock->seqcount >= SEQ_MAX) 
						{
						printf("Config Error in knock %s: too many ports in knock sequence\n",
						knock->name);
						exit(EXIT_FAILURE);
						}								
					protocol = strsep(&num, ":");
					protocol =strtoupper(trim(protocol));
					if(!strcmp(protocol, "TCP"))
						knock->protocol[knock->seqcount] = IPPROTO_TCP;
					else if(!strcmp(protocol, "UDP"))
						knock->protocol[knock->seqcount] = IPPROTO_UDP;
					else if(!strcmp(protocol, "ICMP"))
						knock->protocol[knock->seqcount] = IPPROTO_ICMP;
					else
						{
						printf("Config Error in knock %s: unknown protocol\n",
						knock->name);
						exit(EXIT_FAILURE);
						}
					
					if((port = strsep(&num, ":")))
						{//<if1>
						port = trim(port);
						if (atoi(port)>255)
							{
							printf("Port number in the sequences must be lower than 256 !! Control the conf. file\n");
							exit(EXIT_FAILURE);
							}
						knock->sequence[knock->seqcount] = (u_short)atoi(port);
						}//</if1>
					else
						{
						/* icmp, no destination port */
						knock->sequence[knock->seqcount] = 0;
						}	
					knock->seqcount++;			
					}//</while>	
				continue;				
				}//</parse knock sequence>

			/*parsing tcp flags of the sequence*/
			else if (strncasecmp(line, "tcp_flags=",10)==0)	
				{//<tcp>
				char *flag;
				char *tmp;
				snprintf(line, strlen(line), line+10);
				tmp=line;
				strtoupper(tmp);
				/*can't use directly line with strsep*/
				while((flag = strsep(&tmp, ","))) 
					{//<while>
					/*set flag in knock struct*/
					if(!strcmp(flag,"FIN")) 
						knock->flag_fin = 1;
					else if(!strcmp(flag, "SYN")) 
						knock->flag_syn = 1;
					else if(!strcmp(flag, "RST"))
						knock->flag_rst = 1;
					else if(!strcmp(flag, "PSH"))
						knock->flag_psh = 1;
					else if(!strcmp(flag, "ACK"))
						knock->flag_ack = 1;
					else if(!strcmp(flag, "URG"))
						knock->flag_urg = 1;
					else 
						{
						printf("Config error sequence %s: unrecognized flag \"%s\"\n",	knock->name,flag);
						exit(EXIT_FAILURE);
						}
					#ifdef DEBUG
						printf("Config: tcp flag: %s\n", flag);
					#endif			
					}//</while>
				continue;	
				}//</tcp>
			/*unknown line*/	
			else
				{
				printf("I don't understand line \"%s\"\n", line);
				exit(EXIT_FAILURE);
				}
			}//<parse sequence>
			
		/*error, unknown line*/
		else
			{
			printf("I don't understand line \"%s\"\n", line);
			exit(EXIT_FAILURE);
			}				
		}//</walking conf file>
return;
}



/**********************
** Print program usage
***********************/
void help() 
{
	printf ("\nKnock : knock client licensed under GPL\n");
	printf ("copyright(c)2004 by Beretta Andrea <%sberetta.andrea@libero.it%s>\n", WHITE, DEFAULT);
	printf ("and Riccardo Bianchi <%sikir@iol.it%s>\n", WHITE, DEFAULT);
	printf ("\n");
	printf("usage: knock -d <host> -i <interface>  -s <sequence name> [ -f <conf file> ] [ -g <address> ]\n");
	printf("\noptions:\n");
	printf("  -d,\thostname of the knockserv\n");
	printf("  -f,\tthe configuration file tu use (default is ~/.knockrc)\n");
	printf("  -g,\tthe ip address of the gateway\n");
	printf("  -i,\tnetwork interface to use\n");
	printf("  -s,\tname of the knock sequence to build\n");
	printf("  -h,\tthis help\n");
	printf("\n");
	exit(EXIT_FAILURE);
}



/**********************
** build icmp header 
**********************/
void build_icmp(void)
{
	if (libnet_build_icmpv4_echo
		(
		ICMP_ECHO,	/* type */
		0,			/* code */
		0,			/* sum */
		0x100,		/* id */
		0x100,		/* seq */
		NULL,		/* payload */
		0,			/* payload lenght */
		lnet,		/* libnet context pointer */
		0 			/* We don't need ptag */ 
		)==-1)
			{
			printf ("Libnet Error %s\n", libnet_geterror(lnet));
			exit(EXIT_FAILURE);
			}

	/* options for ip layer */
	h_lenght=LIBNET_ICMPV4_ECHO_H+LIBNET_IPV4_H;
	ip_proto=IPPROTO_ICMP;
	/*build ip header*/
	build_ipv4();
	return;
}



/**********************
**  build upd header 
***********************/
void build_udp(u_short d_port)
{
	if (libnet_build_udp
		(
		rand() % 65535,	/* source port */
		d_port,			/* destination port */
		LIBNET_UDP_H ,	/* size */
		0,				/* sum */
		NULL,			/* payload */
		0,				/* payload size */
		lnet,			/* libnet context */
		0				/* we don't need ptag */
		)==-1)
			{
			printf ("Libnet Error %s\n", libnet_geterror(lnet));
			exit(EXIT_FAILURE);
			}

	/* options for ip layer */
	h_lenght=LIBNET_UDP_H+LIBNET_IPV4_H;
	ip_proto=IPPROTO_UDP;
	/*build ip header*/
	build_ipv4();
	return;
}



/**********************
**  build tcp header
**********************/
void build_tcp(u_short d_port)
{	
	if (libnet_build_tcp 
		(
		rand() % 65535,	/*sport*/
		d_port,			/*dport*/
		847930886, 		/*seq*/
		524972923, 		/*ack*/
		flags,			/*flags*/
		32767,     		/*win*/
		0,         		/*checksum*/
		0,         		/*urg*/
		LIBNET_TCP_H, 	/*header size*/
		NULL,			/*payload*/
		0,				/*payload size*/
		lnet,			/*libnet context*/
		0				/*we don't need ptag*/
		)==-1)
			{
			printf ("Libnet error : %s\n" ,libnet_geterror(lnet));
			exit(EXIT_FAILURE);
			}

	/* options for ip layer */
	h_lenght=LIBNET_TCP_H+LIBNET_IPV4_H;
	ip_proto=IPPROTO_TCP;
	/*build ip header*/
	build_ipv4();
	return;
}



/************************  
**  build ipv4 header 
************************/
void build_ipv4(void)
{
	if (libnet_build_ipv4
	    (
		h_lenght,		/* len */
		0,				/* tos */
		0x200,			/* id */
		0,				/* frag */
		64,				/* ttl */
		ip_proto,		/* transport protocol */
		0,				/* sum */
		IfaceIp,		/* source address */
		KnockServIp,	/* destination address */
		NULL,			/* ip payload */
		0,				/* ip payload size */
		lnet,			/* libnet context */
		0				/* we don't need ptag */
	    )==-1)
			{
			printf ("Libnet Error %s\n", libnet_geterror(lnet));
			exit(EXIT_FAILURE);
	    	}

   /* write packet */
	if (libnet_write(lnet)==-1)
		printf ("Libnet Error %s\n", libnet_geterror(lnet));
	//printf (" Packet correctly wrote on wire\n");
	return;
}



/********************************************
** Map the destination port on the portspan
********************************************/
u_short map(u_short port)
{	
	/*pointer to portspan_list element*/
	PMList *lp=portspan_list;
	u_short diff=port;

	/*scan span list*/
	for( ; lp ; lp=lp->next)
		{//<for>
		port = diff + ((portspan_t*)lp->data)->minrange;
		if (port > ((portspan_t*)lp->data)->maxrange )
			diff=port - ((portspan_t*)lp->data)->maxrange ;
		else
			break;	
		}//</for>
				
	return port;
}



/****************************
** Generate Smokescreen
****************************/
void gen_smoke(void)
{
	int index;
	
	for( index=0 ; index < SMOKE_MAX ; index++)
		{//<for>	
		u_int random=rand();
		u_int port=random %(SMOKE_MAX_PORT - SMOKE_MIN_PORT )+ SMOKE_MIN_PORT ;
		/*verify that the port is out of porspan and correct it*/
		port=check_span(port);	
		smoke_packets[index]=port;
		/*TCP*/
		if(random % 2)		
			smoke_proto[index]=IPPROTO_TCP;
		/*UDP*/
		else
			smoke_proto[index]=IPPROTO_UDP;
		}//</for>	
		
return;
}



/******************************************************
** Check if random port is out of span and correct it
******************************************************/
u_int check_span(u_int port)
{
	/*pointer to portspan_list element*/
	PMList *lp=portspan_list;
	/*pointer to element data*/
	portspan_t *ps;
	
	/*scan span list*/
	for( ; lp ; lp=lp->next)
		{//<for>
		ps=((portspan_t*)lp->data);
		if ( port >= ps->minrange && port <= ps->maxrange )
			{//<if>
			if (ps->maxrange < SMOKE_MAX_PORT)
				port=ps->maxrange+1;
			else
				port=ps->minrange-1;
			}//</if>		
		}//</for>
	return port;	
}



/***************************************
** Put random packets in the knock seq.
****************************************/
void randomize(void)
{	
	int j;
	
	if(smoke_index < SMOKE_MAX  )
		{//<if>
		for(j=0 ; j< SMOKE_NR ; j++)
			{//<for>
			if(rand()%100 < SMOKE_PERC )
				{//<if2>
				#ifdef MODULE_KNOCK
					sleep(1);
				#endif	
				if (smoke_proto[smoke_index]==IPPROTO_UDP)						
					{
					/*UDP*/
					build_udp((u_short)smoke_packets[smoke_index]);
					#ifdef VERBOSE
						printf("[UDP] %d\t\tSmoke packet\n", smoke_packets[smoke_index]);
					#endif
					}
				else
					{
					/*TCP*/
					build_tcp((u_short)smoke_packets[smoke_index]);
					#ifdef VERBOSE
						printf("[TCP] %d\t\tSmoke packet\n", smoke_packets[smoke_index]);
					#endif					
					}
				smoke_index++;	
				}//</if2>
			}//</for>
		}//</if>	
		
	return;
}



/**********************************************
** Open mcrypt module and initialize buffers
**********************************************/
void mcrypt_init()
{
	int i;
	char *salt;
	int salt_size;
	int ivsize;
	KEYGEN data;
	
	/*open module*/
	hd = mcrypt_module_open(CryptAlgName, NULL, "cfb", NULL);
	if (hd==MCRYPT_FAILED)
		{
		printf("Can't init mcrypt module\n");
		exit(EXIT_FAILURE);
		}
		
	/* get algorithm key size */
	keysize = mcrypt_enc_get_key_size(hd);
	if (hd==MCRYPT_FAILED)
		{
		printf("Can't open mcrypt module\n");
		exit(EXIT_FAILURE);
		}
	
	/*generate the key using the password*/
	if (mhash_get_keygen_salt_size(KEYGEN_MCRYPT)==0) 
		salt_size=10;
	else
		salt_size = mhash_get_keygen_salt_size(KEYGEN_MCRYPT);
	salt = calloc( 1, salt_size);
	Key = calloc( 1, keysize);			
	data.hash_algorithm[0] = MHASH_MD5;
	data.count = 0;
	data.salt = salt;
	data.salt_size = salt_size;
	mhash_keygen_ext(KEYGEN_MCRYPT, data, Key, keysize, password, strlen(password));	
	
	/*generate the iv using the temp_iv*/
	if(mcrypt_enc_mode_has_iv(hd)) 
		{
		/*get iv size*/	
		ivsize=mcrypt_enc_get_iv_size(hd);	
		IV = calloc( 1, ivsize);
		mhash_keygen_ext(KEYGEN_MCRYPT, data, IV, ivsize, temp_iv, strlen(temp_iv));
		}
	
	if (hd==MCRYPT_FAILED)
		{
		printf("Can't open mcrypt module\n");
		exit(EXIT_FAILURE);
		}	
		
	i=mcrypt_generic_init( hd, Key, keysize, IV);
	if (i<0) 
		{
		mcrypt_perror(i);
		exit(EXIT_FAILURE);
  		}
  	return ;
}

