/*
 *  knock.h
 * 
 *  Copyright (c) 2004 by Beretta Andrea <beretta.andrea@libero.it>
 *  and Bianchi Riccardo <ikir@iol.it>
 *
 *  Thanks to Marco Balduzzi <embyte@spine-group.org>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *
 */

/*includes*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <arpa/inet.h>	
#include <libnet.h>
/*time*/
#include <time.h>
#include <bits/time.h>
/*libmcrypt encryption/decryption library*/
#include <mcrypt.h>
/*hashing*/
#include <mhash.h>
/*misc ops*/
#include "misc.h"
/*operations on list*/
#include "list.h"



/*constant values*/
/*#define DEBUG*/
#define VERBOSE
/*#define MODULE_KNOCK*/
/*escape sequences*/
#define WHITE "\033[1;37m"
#define DEFAULT "\033[0m"
/*max knock sequence length*/
#define SEQ_MAX 15			
/*max nr. of smoke elements between right packets*/
#define SMOKE_NR 3
/*smoke elements to create*/
#define SMOKE_MAX SEQ_MAX*SMOKE_NR		
/*prob. of a smoke packet to be included in the sequence*/
#define SMOKE_PERC 60
/*max port number in a smoke packet, max 65535*/
#define SMOKE_MAX_PORT 15000
/*min port number in a smoke packet, min 1024*/
#define SMOKE_MIN_PORT 1024
/*source port used for the packets we're going to create*/
#define SOURCE_PORT 10000



/* knock/event struct */
typedef struct knock_struct {
	char name[128];
	u_short seqcount;
	u_char sequence[SEQ_MAX];
	u_short protocol[SEQ_MAX];
	u_short flag_fin;
	u_short flag_syn;
	u_short flag_rst;
	u_short flag_psh;
	u_short flag_ack;
	u_short flag_urg;
} knock_t;


/* port span struct*/
typedef struct portspan {
	u_int minrange;
	u_int maxrange;
} portspan_t;



/*Global Vars*/
knock_t *knock=NULL;
portspan_t *port_range=NULL;
/*list of various knock*/
PMList *knocks_list = NULL;
/*list of ranges in portspan*/
PMList *portspan_list=NULL;
/*cypher*/
char CryptAlgName[100];
char temp_iv[16];
u_char *IV=NULL;		
/*password used to generate the key*/
char password[256];
/*key used to crypt*/
u_char *Key;		
/*key size*/	
int keysize;		
/* libnet context handler */
libnet_t *lnet;
/* Mcrypt encryption descriptor*/
MCRYPT hd; 	
/*ip of our interface*/
u_long IfaceIp;			
/*ip of the knockserv*/
u_long KnockServIp;		
/* header lenght */
u_short h_lenght;		
/* ip-proto ip header field */
u_short ip_proto;		
/*tcp flags*/
int flags = 0x00;		
/*smoke screen packets*/
u_int smoke_packets[SMOKE_MAX];	
/*smoke screen proto*/
u_int smoke_proto[SMOKE_MAX];	
/*smoke arrays index*/
int smoke_index=0;



/* function prototypes */
int main(int argc, char** argv);
/* Print program usage */
void help();
/*parse the config file*/
void parse_config(char *filename);
/*  build icmp header */
void build_icmp(void);
/*  build udp header */
void build_udp(u_short d_port);
/*  build tcp header */
void build_tcp(u_short d_port);
/*  build ipv4 header */
void build_ipv4(void);
/* Map the destination port on the portspan */
u_short map(u_short port);
/*Generate Smokescreen*/
void gen_smoke(void);
/* Check if random port is out of span and correct it */
u_int check_span(u_int port);
/*Put random packets in the knock seq.*/
void randomize(void);
/*Open mcrypt module and initialize buffers*/
void mcrypt_init(void);
