/*
 * servknock.c
 *
 *  Copyright (c) 2004 by Beretta Andrea <beretta.andrea@libero.it>
 *  and Bianchi Riccardo <ikir@iol.it>
 *
 *  Thanks to Judd Vinet <jvinet@zeroflux.org>
 *  and Paolo Carpo <snifth@spine-group.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 */

/* includes */
#include "servknock.h"


/**********************
** Print program usage
***********************/
void give_help()
{
	printf ("ServKnock : portknocking server licensed under GPL\n");
	printf ("copyright(c)2004 by Beretta Andrea <%sberetta.andrea@libero.it%s>\n", WHITE, DEFAULT);
	printf ("and Riccardo Bianchi <%sikir@iol.it%s>\n", WHITE, DEFAULT);
	printf ("\n");
	printf ("usage: servknock [OPTION] [ -f <conf_file> ]\n");
	printf ("\noptions : \n");
	printf ("  -d,\trun as daemon\n");
	printf ("  -D,\tprint debug info\n");
	printf ("  -f,\talternative conf. file (default is /etc/servknock.conf)\n");
	printf ("  -h,\tthis help\n");
	printf ("  -V,\tverbose\n\n");
	exit(EXIT_FAILURE);
}



/*******************
** Main function
*******************/
int main(int argc, char **argv)
{
	/* command line options*/
	int option;
	int daemon_mode=0;
	/*configuration file*/
	char *conf_file;	
	/* netmask */
	bpf_u_int32 maskp;		
	/* net addr. */
	bpf_u_int32 netp;		
	/* pointer to bpf_program struct, used in filters*/	
	struct bpf_program fp;	
	/* struct filled in by pcap_next() */
	struct pcap_pkthdr hdr;
	/* offset for protocol headers */
	int offset;	
	/* our packet */
	char *packet;	
	/* type of datalink we're goint to use */
	int datalink;	
	/* our bpf filter */	
	char *filter;	
	/* error buffer */
	char errbuf[PCAP_ERRBUF_SIZE];	
	/*var used for keygen*/
	char *salt;
	int salt_size;
	int ivsize;
	KEYGEN data;

	filter = packet = conf_file = NULL;
	
	/*only root can run the program*/
	if( getuid() )
		{
		printf("You should be root to run this program\n");
		exit(EXIT_FAILURE);
		}
	
	/* parse cmdline */
	while ((option = getopt (argc, argv, "f:dhVD?"))!=EOF)
		{//<while>
		switch (option)
			{//<switch>
			case 'd':
				daemon_mode=1;
				break;
			case 'D':
				opt_debug=1;
				break;
			case 'f':
				conf_file=malloc(strlen(optarg)+1);
				strcpy(conf_file, optarg);
				break;								 								
			case 'V':
				opt_verbose=1;
				break;
			case 'h':
			case '?':
			default: /*Warning falling code*/
				give_help();
				break;
			}//</switch>
		}//</while>
	
	/*if in daemon mode fork the process and run it in bg*/
	if(daemon_mode)
		{//<daemon>
		int pid;
		if( opt_debug || opt_verbose )
			{
			printf("You can't use debug or verbose options with daemon mode\n");
			exit(EXIT_FAILURE);
			}

		if( (pid=fork()) < 0 )
			{
			printf("Error can't fork process\n");
			exit(EXIT_FAILURE);
			}
		/*children*/	
		if (pid==0)
			{
			/*create new session*/
			setsid();
			/*close streams ?*/
			close(0);
			close(1);
			close(2);
			}
		/*father*/
		else 
			{
			printf("ServKnock, running in background\n");
			exit(EXIT_SUCCESS);
			}			
		}//</daemon>

	/*if not specified we use standard conf_file*/	
	if(conf_file==NULL)
		{//<conf file>
		char *std_conf="/etc/servknock/servknock.conf";
		conf_file=malloc(strlen(std_conf)+1);
		strcpy(conf_file, std_conf);	
		}//</conf_file>		
	
	if(opt_debug)
		printf("The configuration file is : %s\n", conf_file);
	
	/*parse the config file*/
	parse_config(conf_file);
	
	/*open mcrypt module*/
	hd = mcrypt_module_open(CryptAlgName, NULL, "cfb", NULL);
	if (hd==MCRYPT_FAILED)
		{
		printf("Can't open mcrypt module\n");
		exit(EXIT_FAILURE);
		}
		
	/* get algorithm key size */
	keysize = mcrypt_enc_get_key_size(hd);
	if (hd==MCRYPT_FAILED)
		{
		printf("Can't open mcrypt module\n");
		exit(EXIT_FAILURE);
		}

	/*generate the key using the password*/
	if (mhash_get_keygen_salt_size(KEYGEN_MCRYPT)==0) 
		salt_size=10;
	else
		salt_size = mhash_get_keygen_salt_size(KEYGEN_MCRYPT);
	salt = calloc( 1, salt_size);
	Key = calloc( 1, keysize);			
	data.hash_algorithm[0] = MHASH_MD5;
	data.count = 0;
	data.salt = salt;
	data.salt_size = salt_size;
	mhash_keygen_ext(KEYGEN_MCRYPT, data, Key, keysize, password, strlen(password));		
		
	/*generate the iv using the temp_iv*/
	if(mcrypt_enc_mode_has_iv(hd))
		{
		/*get iv size*/
		ivsize=mcrypt_enc_get_iv_size(hd);
		IV= calloc(1, ivsize);
		mhash_keygen_ext(KEYGEN_MCRYPT, data, IV, ivsize, temp_iv, strlen(temp_iv));
		}
	
	/* signal handlers */
	signal(SIGQUIT, sigexit);
	signal(SIGTERM, sigexit);
	signal(SIGINT, sigexit);
	signal(SIGABRT, sigexit);
	signal(SIGCHLD, sigzombie);

	/* check if the dev is up and read its parameters */  
	if ((pcap_lookupnet(IfaceName,&netp,&maskp,errbuf))==-1)
		{
		printf ("Error: pcap_lookupnet(): %s\n\n", errbuf);
		exit(EXIT_FAILURE);
		}
   	
	/* open our interface NOT in promisc mode and without timeout */
	if ((descr = pcap_open_live (IfaceName, MAX_BYTES, 0, 500, errbuf))==NULL)
		{
		printf("Error: pcap_open_live(): %s\n\n",errbuf);
		exit(EXIT_FAILURE);
		}

	/* look for our datalink and get our offset */
	if ((datalink = pcap_datalink(descr)) < 0)
		{
		printf("Error: pcap_datalink(): %s\n\n", errbuf);
		exit(EXIT_FAILURE);
		}
		
	switch (datalink)
		{//<switch>
		case DLT_EN10MB:	/*ethernet*/
			offset = 14;
			break;
		case DLT_NULL:
		case DLT_PPP:		/*ppp*/
			offset = 4;
			break;
		case DLT_SLIP:		/*serial line*/
			offset = 16;
			break;
		case DLT_RAW:		/*raw ip packet*/
			offset = 0;
			break;
		case DLT_SLIP_BSDOS:
		case DLT_PPP_BSDOS:
			offset = 24;
			break;
		case DLT_LINUX_SLL:	/* Linux "cooked" capture encapsulation */
			offset = 16;
			break;
		default:
			printf("Error: Unknown Datalink Type: (%d)\n\n", datalink);
			exit(EXIT_FAILURE);
     	}//</swhitch>

	/* compile the filter */
	if(pcap_compile(descr,&fp,filter,0,netp) == -1)
		{
		printf( "Error calling pcap_compile()\n\n");
		exit(EXIT_FAILURE);
		}

	/* set the filter */
	if(pcap_setfilter(descr,&fp) == -1)
		{
		printf("Error calling pcap_setfilter()\n\n");
		exit(EXIT_FAILURE);
		}
		
	pcap_freecode(&fp);	
	
	if (! daemon_mode)
		printf("\nStart sniffing on %s\n", IfaceName);
		
	/* read packets */
	while(1)
		{	
		/* read with pcap_next() */
		if ( (packet = (char *) pcap_next (descr, &hdr)) == NULL) 
			continue;
		/*process the packet*/	
		proc_packet(packet, offset, &hdr);		
		}
	
	exit(EXIT_SUCCESS);
}



/*******************
** signal handler 
*******************/
void sigexit(void)
{
if (LogFilePtr)
	{//<if>
	fprintf(LogFilePtr, "\nServKnock shutdown\t[OK]\n");
	fprintf(LogFilePtr,"\n********\n"); 
	fclose(LogFilePtr);
	}//</if>
/*close pcap_descriptor*/  	
pcap_close(descr);
/*close mcrypt module*/
mcrypt_module_close	(hd);
   
exit(EXIT_SUCCESS);
}



/*******************
** remove zombies
*******************/
void sigzombie(void)
{
 while(waitpid(WAIT_ANY, NULL, WNOHANG) > 0)
 	;
 return;
}


/*****************************
** Parse config file
****************************/
void parse_config (char *filename)
{
	FILE *fp = NULL;	
	char LogFileKnock[1024];	/*name of the logfile*/
	char *line=malloc(PATH_MAX + 1);
	char *tmp_line;
	int in_seq_flag=0; 		/*flag used to parse seq*/
	int range_count=0;		/*n. of ports in a range of portspan*/
	
	/*open the file*/
	if((fp = fopen(filename, "r")) == NULL)
		{
		printf("Cannot open default config file %s, specify another filname\n\n", filename);
		give_help();
		}
	
	while(fgets(line, PATH_MAX, fp)) 
		{//<walking conf file>
		/*trim spaces and newline*/
		trim(line);
		/*ignore comment line*/
		if(!strlen(line) || *line=='#')
			continue;
			
		/* interface */
		if (strncasecmp(line, "iface=",6)==0)
			{//<iface>
			int err;
			snprintf(IfaceName, 20, line+6);
			if(opt_debug)
				printf("\nListening interface \"%s\"\n",IfaceName);

					
			err = get_interface_address (IfaceName, IfaceIp, sizeof(IfaceIp));
			if (err) 
				{
        		printf("Can't get interface address of %s\n", IfaceName);
		        exit(EXIT_FAILURE) ;
		    	}
			if(opt_debug)
				printf("Ip interface %s\n", IfaceIp);		
			continue;
			}//</iface>
						
		/* cypher */
		else if (strncasecmp(line, "cypher=",7)==0)
			{//<cypher>
			snprintf(CryptAlgName, 100, line+7);
			if(opt_debug)
				printf("Crypt Algorithm is \"%s\"\n",CryptAlgName);
			continue;	
			}//</cypher>

		/* password */	
		else if (strncasecmp(line, "password=",9)==0)
			{//<key>
			snprintf(password, 16, line+9);				
			//snprintf(Key, 16, line+9);									
			if(opt_debug)
				printf("The password is \"%s\"\n",password);
			continue;	
			}//</key>
			
		/*Inizialization Vector*/
		else if (strncasecmp(line, "iv=",3)==0)
			{//<key>
			snprintf(temp_iv, 16, line+3);
			if(opt_debug)		
				printf("\nThe IV is \"%s\"\n",temp_iv);
			continue;
			}//</key>					

		/* portspan */
		else if (strncasecmp(line, "span=",5)==0)
			{//<portspan>
			char *temp;
			char *min;
			char *max;
			snprintf(line, strlen(line), line+5);
			tmp_line=line;				
			if(opt_debug)
				printf("The port span is ");
	
			while((temp = strsep(&tmp_line, ","))) 
				{//<while>
				port_range = malloc(sizeof(portspan_t));
				if(port_range == NULL) 
					{
					printf("Error: can't alloc portspan structure\n");
					exit(EXIT_FAILURE);
					}
				min = strsep(&temp, "-");
			 	min =trim(min);
			 	port_range->minrange=atoi(min);
			 	max= strsep(&temp, "-");
			 	max =trim(max);	
			 	port_range->maxrange=atoi(max);
				range_count+=(port_range->maxrange - port_range->minrange);
			 	/*
			 	if (range_count!=255)
			 		{
			 		printf("\nPortspan ranges != 255 ports\n");
			 		exit(EXIT_FAILURE);
			 		}
			 	*/	 	
				portspan_list=list_add(portspan_list, port_range);
				if(opt_debug)
					printf("%d) %d-%d\t",list_count(portspan_list) ,port_range->minrange,port_range->maxrange);	
				}//</while>
			
			if (range_count!=255)
			 	{
			 	printf("\nPortspan ranges != 255 ports\n");
			 	exit(EXIT_FAILURE);
			 	}
			 		
			if(opt_debug)
				printf("\n");
			continue;
			}//</portspan>
	
		/* log	*/
		else if (strncasecmp(line, "LogFile=",8)==0)
			{//<log>
			char *start_log_line;
			snprintf(LogFileKnock, 256, line+8);
			if(opt_debug)
				printf("Logfile is \"%s\"\n",LogFileKnock);
			LogFilePtr=fopen(LogFileKnock,"a");
			if (!LogFilePtr)
				{
				printf("Couldn't open %s for appending\n",LogFileKnock);
				exit(EXIT_FAILURE);
				}
			setbuf(LogFilePtr,NULL);	
			start_log_line=malloc(strlen(LogFileKnock)+ 10);
			strcpy(start_log_line, "date >> ");
			strcat(start_log_line, LogFileKnock);
			system(start_log_line);		
			fprintf(LogFilePtr, "Start sniffing on %s (%s)\n", IfaceName, IfaceIp );	
			continue;
			}//</log>
													
		/* start seq */
		else if (strncasecmp(line, "<seq>",5)==0)
			{//<found new seq>
			in_seq_flag=1;
			knock = malloc(sizeof(knock_t));
			if(knock == NULL) 
				{
				printf("Error: can't alloc knock structure\n");
				exit(EXIT_FAILURE);
				}
			continue;
			}//</found new seq>

		/* parsing seq data */	
		else if(in_seq_flag)
			{//<parse sequence>
	
			/* end of current seq */
			if(strncasecmp(line, "</seq>",6)==0)
				{//<end seq>
				if(opt_debug)
					{//<opt debug>
					int i;		
					printf("knock packets\n");
					for(i = 0; i < knock->seqcount; i++) 
						{//<for>
						switch(knock->protocol[i])
							{//<switch>
							case IPPROTO_UDP:
								printf("\t%d) %d : udp\n", i+1, knock->sequence[i]);
								break;
							case IPPROTO_TCP: 
								printf("\t%d) %d : tcp\n", i+1, knock->sequence[i]);
								break;
							case IPPROTO_ICMP:
								printf("\t%d) ICMP\n", i+1);
								break;
							}//</switch>
						}//</for>
					}//</opt_debug>
				in_seq_flag=0;
				continue;
				}//</end seq>
	
			/* seq name */
			else if(strncasecmp(line, "name=",5)==0)
				{//<name>
				snprintf(knock->name, sizeof(knock->name), line+5);		
				knock->seqcount = 0;
				knock->seq_timeout  = SEQ_TIMEOUT; 
				knock->start_command_list = NULL;
				knock->cmd_timeout = CMD_TIMEOUT; 
				knock->stop_command_list = NULL;
				knocks_list = list_add(knocks_list, knock);
				if(opt_debug)
					printf("\nSequence %d) %s\n ",list_count(knocks_list), knock->name);
				continue;
				}//</name>
			
			/* parsing knock element */
			else if(strncasecmp(line, "sequence=",9)==0)
				{//<parse elements>
				char *num;
				char *protocol;
				char *port;
		
				snprintf(line, strlen(line), line+9);
				tmp_line=line;
				while((num = strsep(&tmp_line, ","))) 
					{//<while>
					if(knock->seqcount >= SEQ_MAX) 
						{
						printf("Config Error in knock %s: too many ports in knock sequence\n", knock->name);
						exit(EXIT_FAILURE);
						}								
					protocol = strsep(&num, ":");
					protocol =strtoupper(trim(protocol));
					if(!strcmp(protocol, "TCP"))
						knock->protocol[knock->seqcount] = IPPROTO_TCP;
					else if(!strcmp(protocol, "UDP"))
						knock->protocol[knock->seqcount] = IPPROTO_UDP;
					else if(!strcmp(protocol, "ICMP"))
						knock->protocol[knock->seqcount] = IPPROTO_ICMP;
					else
						{
						printf("Config Error in knock %s: unknown protocol\n",
						knock->name);
						exit(EXIT_FAILURE);
						}
						
					if((port = strsep(&num, ":")))
						{//</if port>
						port = trim(port);
						knock->sequence[knock->seqcount] = (u_short)atoi(port);
						}//</if>						
					/* icmp, no destination port */
					else
						knock->sequence[knock->seqcount] = 0;
					knock->seqcount++;			
					}//</while>				
				continue;				
				}//</parse elements>

			/* seq. timeout	*/
			else if (strncasecmp(line, "seq_timeout=",12)==0)
				{//<timeout>
				snprintf(line, strlen(line), line+12);
				knock->seq_timeout=(time_t) atoi(line);
				if(opt_debug)
					printf("Seq timeout is \"%d\"\n",(int) knock->seq_timeout);
				continue;				
				}//</timeout>	

			/* command timeout */
			else if (strncasecmp(line, "cmd_timeout=",12)==0)
				{//<cmd timeout>
				snprintf(line, strlen(line), line+12);
				knock->cmd_timeout=(time_t) atoi(line);
				if(opt_debug)
					printf("Cmd timeout is \"%d\"\n",(int) knock->cmd_timeout);
				continue;	
				}//</cmd timeout>	
			
			/* parsing tcp flags */
			else if (strncasecmp(line, "tcp_flags=",10)==0)	
				{//<tcp>
				char *flag;
				snprintf(line, strlen(line), line+10);
				strtoupper(line);
				tmp_line=line;
				while((flag = strsep(&tmp_line, ","))) 
					{//<while>
					if(!strcmp(flag,"FIN")) 
						knock->flag_fin = 1;
					else if(!strcmp(flag, "SYN")) 
						knock->flag_syn = 1;
					else if(!strcmp(flag, "RST"))
						knock->flag_rst = 1;
					else if(!strcmp(flag, "PSH"))
						knock->flag_psh = 1;
					else if(!strcmp(flag, "ACK"))
						knock->flag_ack = 1;
					else if(!strcmp(flag, "URG"))
						knock->flag_urg = 1;
					else 
						{
						printf("Config error sequence %s: unrecognized flag \"%s\"\n",	knock->name,flag);
						exit(EXIT_FAILURE);
						}
					if(opt_debug)
						printf("Config: tcp flag: %s\n", flag);
					}//</while>
				continue;					
				}//</tcp>
		
			/* start command */
			else if (strncasecmp(line, "command=",8)==0)
				{//<start command>
				snprintf(line, strlen(line), line+8);
				command = malloc(sizeof(command_t));
				if(command == NULL) 
					{
					printf("Error: can't alloc command structure\n");
					exit(EXIT_FAILURE);
					}
				command->bin=malloc(strlen(line)+1);
				snprintf(command->bin, strlen(line)+1, line);		
				knock->start_command_list = list_add(knock->start_command_list, command);
				if(opt_debug)
					printf("start command %d) %s\n", list_count(knock->start_command_list), command->bin);
				continue;				
				}//</start command>
				
			/* stop command */
			else if (strncasecmp(line, "stop_command=",13)==0)
				{//<command>
				snprintf(line, strlen(line), line+13);
				command = malloc(sizeof(command_t));
				if(command == NULL) 
					{
					printf("Error: can't alloc command structure\n");
					exit(EXIT_FAILURE);
					}
				command->bin=malloc(strlen(line)+1);
				snprintf(command->bin, strlen(line)+1, line);		
				knock->stop_command_list = list_add(knock->stop_command_list, command);
				if(opt_debug)
					printf("stop command %d)  %s\n", list_count(knock->stop_command_list),command->bin);
				continue;				
				}//</stop command>	
			/* unknown line */	
			else
				{
				printf("I don't understand line \"%s\"\n", line);
				exit(EXIT_FAILURE);
				}				
			}//<parse seq
		
		/* error, unknown line */
		else
			{
			printf("I don't understand line \"%s\"\n", line);
			exit(EXIT_FAILURE);
			}																						
		}//</walking conf file>		
		
return;		
}



/***************************************
* Called with every packet, process it
****************************************/
void proc_packet(char *packet, int offset, struct pcap_pkthdr* hdr)
{
	/* pointer to ipv4 header */
	struct ipv4_hdr *ip=NULL;	
	/*pointer to tcp header*/
   	struct tcp_hdr *tcp=NULL;	
   	/*pointer to udp header*/
    struct udp_hdr *udp=NULL;
	/*Source Ip, Destination ip and port*/
	u_short	DPort;	
	char IPSrc[16]; 
	char IPDst[16];
	/*vars used with strsep, working on ipbyte*/
	char* tmp_ip=IPSrc;
	char* byte;
	int i=0;
	/*pointer used to scan lists*/
	PMList* lp;

	/* timestamp */
	struct tm* pkt_tm;
	time_t pkt_secs;	/* /usr/include/bits/time.h */
	char pkt_date[11];
	char pkt_time[9];
	
	ip = (struct ipv4_hdr *) (packet+offset);
	
	/*check ip version, if not ipv4 ignore*/
	if(ip->ip_v != 4) 
		return;
	
	/*if we aren't the destination ip, ignore the packet*/
	strncpy(IPSrc, inet_ntoa(ip->ip_src),sizeof(IPSrc) );
	strncpy(IPDst, inet_ntoa(ip->ip_dst),sizeof(IPSrc) );
	if (strcmp(IPDst, IfaceIp))
		return;
		
	/* timestamp*/
	pkt_secs = hdr->ts.tv_sec;
	pkt_tm = localtime(&pkt_secs);
	snprintf(pkt_date, 11, "%04d-%02d-%02d", pkt_tm->tm_year+1900, pkt_tm->tm_mon,
			pkt_tm->tm_mday);
	snprintf(pkt_time, 9, "%02d:%02d:%02d", pkt_tm->tm_hour, pkt_tm->tm_min,
				pkt_tm->tm_sec);
	
	/*check protocol*/
	switch(ip->ip_p)
		{//<switch>
		/*UDP packet*/
		case IPPROTO_UDP:
			udp = (struct udp_hdr *) (packet + (ip->ip_hl * 4) + offset);
			/*if the destination port is out of portspan, ignore the udp packet*/		
			DPort=ntohs(udp->uh_dport);
			if (!(lp=check_port_range (DPort)))
				return;
			DPort = demap(DPort,lp);
			if(opt_debug)
				printf("[UDP] port %d\n", DPort );
			break;
		/*ICMP packet*/
		case IPPROTO_ICMP:
			DPort = 0;
			if(opt_debug)
				printf("[ICMP] icmp packet\n");
			break;
		/*TCP packet*/
		case IPPROTO_TCP:
			tcp = (struct tcp_hdr *) (packet + (ip->ip_hl * 4) + offset);		   
			/*if the destination port is out of portspan, ignore the tcp packet*/
			DPort= ntohs(tcp->th_dport);
			if (!(lp=check_port_range (DPort)))
				return;
			DPort=demap(DPort,lp);
			if(opt_debug)
				{
				printf("[TCP] port %d\n", DPort );
				printf("FIN(%d) SYN(%d) RST(%d) PSH(%d) ACK(%d) URG(%d)\n", tcp->fin, tcp->syn, tcp->rst, tcp->psh, tcp->ack, tcp->urg );
				}
			break;
		/*other protocols, ignore them*/		     
		default:
			return;
		}//</switch>
	  	  	  	  
	/* clean up expired/completed/failed attempts */
	for (lp=attempts_list; lp; lp=lp->next) 
		{//<for cleanup>
		int delete=0;
		attempt=(attempt_t*)lp->data;
		
		/*succesful attempt found*/		
		if (attempt->ipstage >= 4) 
			{
			if(opt_verbose)
				{
				printf ("[%s] Removing successful knock attempt from %s", attempt->knock->name, attempt->src);
				printf("\t[%sDELETED%s]\n", RED, DEFAULT);
				}
			delete=1;
			}
		/*failed attempt found*/
		else if (attempt->stage == -1 || attempt->ipstage == -1) 
			{
			if(opt_verbose)
				{
				printf ("[%s] Removing failed knock attempt from %s",attempt->knock->name, attempt->src);
				printf("\t[%sDELETED%s]\n", RED, DEFAULT);
				}
			fprintf(LogFilePtr, "[%s] %s-%s Failed knock attempt from ip %s\n", attempt->knock->name, pkt_date, pkt_time, attempt->src );									
			delete = 1;
			}
		/*expired attempt found*/				
		else if ((pkt_secs - attempt->seq_start) >= attempt->knock->seq_timeout) 
			{
			if(opt_verbose)
				{
				printf ("[%s] Seq timeout from %s", attempt->knock->name, attempt->src );
				printf("\t[%sDELETED%s]\n", RED, DEFAULT);
				}
			fprintf(LogFilePtr, "[%s] %s-%s Seq timeout from ip %s\n", attempt->knock->name, pkt_date, pkt_time, attempt->src );						
			delete=1;
			}
			
		/*clean list*/	
		if (delete) 
			{//<delete>			
			/* splice this entry out of the list */
			if(lp->prev) lp->prev->next = lp->next;
			if(lp->next) lp->next->prev = lp->prev;
			if(lp == attempts_list) attempts_list = NULL;
			lp->prev = lp->next = NULL;
			list_free(lp);
			continue;
			}//</delete>
		}//</for cleanup>
	
	attempt=NULL;
	
	/* check if the packet is part of existing attempt*/
	for (lp=attempts_list; lp; lp=lp->next) 
		{//<for exist>
		if (!strncmp(((attempt_t*)lp->data)->src,IPSrc,sizeof(IPSrc))) 
			{
			/*found attempt*/
			attempt=(attempt_t*)lp->data;
			break;			
			}
		}//</for exist>
	
	if (attempt) 
		{//<update state of current attempt>
		int flagsmatch = 1;
		/* if TCP packet check its flags*/
		if(tcp!=NULL) 
			{//<check tcp flags>
			if(attempt->knock->flag_fin == 1 && tcp->fin != 1) 
				{
				if(opt_debug)
					printf("[%s] packet is not FIN, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			else if(attempt->knock->flag_syn == 1 && tcp->syn != 1) 
				{
				if(opt_debug)
					printf("[%s] packet is not SYN, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			else if(attempt->knock->flag_rst == 1 && tcp->rst != 1) 
				{
				if(opt_debug)
					printf("[%s] packet is not RST, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			else if(attempt->knock->flag_psh == 1 && tcp->psh != 1) 
				{
				if(opt_debug)
					printf("[%s] packet is not PSH, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			else if(attempt->knock->flag_ack == 1 && tcp->ack != 1) 
				{
				if(opt_debug)
					printf("[%s] packet is not ACK, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			else if(attempt->knock->flag_urg == 1 && tcp->urg != 1) 
				{	
				if(opt_debug)
					printf("[%s] packet is not URG, ignoring...\n", attempt->knock->name );
				flagsmatch = 0;
				}
			}//</check tcp flags>
		
		/*if flag don't match ignore this packet*/
		if (flagsmatch == 0)
			return; 
			
		/*it the first part of the knock sequence is OK , check for the ip*/	
		if (attempt->is_in_ipstage) 
			{//<ipstage>
			/*check the ipstage byte of the ip address*/
			if (DPort == attempt->ipbyte[attempt->ipstage]) 
				{//<check the byte>
				/* ip byte is OK!! */
				attempt->ipstage++;
				if(opt_debug)
					printf("[%s] %s Stage %d\n", attempt->knock->name, attempt->src, attempt->stage+attempt->ipstage);
				fprintf(LogFilePtr, "[%s] %s-%s Stage %d from ip %s\n", attempt->knock->name, pkt_date, pkt_time, attempt->stage+attempt->ipstage ,attempt->src );					
				/*succesful knock*/
				if(attempt->ipstage >= 4) 
					{//<successful knock>
					PMList *temp;
					command_t *execute;
					/*knock is complete*/
					if(opt_verbose)
						printf("[%s] Knock sequence completed\n", attempt->knock->name);									
					fprintf(LogFilePtr, "[%s] %s-%s Successful knock attempt from ip %s\n", attempt->knock->name, pkt_date, pkt_time, attempt->src );				
					execute=(command_t*)attempt->knock->start_command_list->data;
					
					/*if exist a command in command_t struct execute it*/
					if(execute->bin && strlen(execute->bin))
						{//<exist at least 1 command>
						/*fork the process*/						
						if (fork()==0)
							{//<fork>
							/* child */
							setsid();
							/* execute the commands in command_list */
							for(temp=attempt->knock->start_command_list ; temp ; temp=temp->next)
								{
								execute=(command_t*)temp->data;
								if(opt_debug)
									printf("[%s] command :%s\n", attempt->knock->name ,execute->bin);
								/*exec*/
								exec_command(parse_commad(execute->bin, attempt->src), attempt->knock->name, pkt_date, pkt_time);																
								}
								
							/*if exist stop commands execute them*/
							if( (execute=(command_t*)attempt->knock->stop_command_list->data) )
								{//<stop>
								if(execute->bin && strlen(execute->bin))
									{
									/*wait cmd_timeout*/
									sleep(attempt->knock->cmd_timeout);
									fprintf(LogFilePtr, "[%s] %s-%s Command timeout : %d \n",attempt->knock->name, pkt_date, pkt_time, (int) attempt->knock->cmd_timeout );
									/*scan command list*/
									for(temp=attempt->knock->stop_command_list ; temp ; temp=temp->next)
										{//<for stop>
										execute=(command_t*)temp->data;
										if(opt_debug)
											printf("[%s] stop command :%s\n", attempt->knock->name, execute->bin);									
										/*run command*/									
										exec_command(parse_commad(execute->bin, attempt->src), attempt->knock->name, pkt_date, pkt_time);										
										}//</for stop>																											
									}
								}//</stop>								
							/*exit child*/	
							exit(EXIT_SUCCESS);
							}//</fork>
						}//<exist 1 command>																																
					}//</successful knock>
				}//</check the byte> 
			/*ip byte is wrong, seq. failed*/
			else 
				attempt->ipstage = -1; 
			}//</ipstage> 
			
		/*we are in the first part of the knock still*/
		else 
			{//<first part of knock>
			/*check protocol and destination port*/
			if( ip->ip_p == attempt->knock->protocol[attempt->stage] && DPort == attempt->sequence_crypted[attempt->stage]) 
				{//<check proto&port>
				attempt->stage++;
				if(opt_debug)
					printf("[%s] %s: Stage %d\n",  attempt->knock->name, attempt->src, attempt->stage);				
				fprintf(LogFilePtr, "[%s] %s-%s Stage %d from ip %s\n",attempt->knock->name, pkt_date, pkt_time, attempt->stage, attempt->src );
				if(attempt->stage >= attempt->knock->seqcount) 
					{
					/*first part of the sequence is OK!*/
					if(opt_debug)
						printf("[%s] First part of knock completed\n", attempt->knock->name );
					fprintf(LogFilePtr, "[%s] %s-%s First part of knock completed from ip %s\n",attempt->knock->name, pkt_date, pkt_time, attempt->src );					
					attempt->is_in_ipstage = 1;
					}
				}//<check proto&port> 
			else 
				{
				/*check for proto&port has failed, knock sequence is wrong*/
				attempt->stage = -1; 
				}
			}//</prima parte knock>
		
		}//</update state of current attempt> 

	/*processing the first packet of a sequence*/
	else 
		{//<first packet of the sequence 
		/* Detect the sequence by decrypting the first port */
		if (ip->ip_p !=  IPPROTO_ICMP) 
				{
				/*if decrypt fails, ignore this packet*/
				if( decrypt_first((u_char*)&DPort) )
					return;
				}
		
		/*scan knock list*/		
		for(lp=knocks_list; lp; lp=lp->next) 
			{//<walk knock list>
			knock=(knock_t*)lp->data;
			
			/*port&proto check*/
			if(ip->ip_p != knock->protocol[0] || DPort != knock->sequence[0]) 
				{
				if(opt_debug);
					printf("Bad start packet for %s, nothing to do\n", knock->name);
				continue;
				}
			
			/* flag check */
			if (tcp) 
				{//<check tcp flags>
				if(knock->flag_fin == 1 && tcp->fin != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not FIN, ignoring...\n", knock->name); 
					continue;
					}
				if(knock->flag_syn == 1 && tcp->syn != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not SYN, ignoring...\n", knock->name); 
					continue;
					}
				if(knock->flag_rst == 1 && tcp->rst != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not RST, ignoring...\n", knock->name); 
					continue;
					}
				if(knock->flag_psh == 1 && tcp->psh != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not PSH, ignoring...\n", knock->name); 
					continue;
					}
				if(knock->flag_ack == 1 && tcp->ack != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not ACK, ignoring...\n", knock->name); 
					continue;
					}
				if(knock->flag_urg == 1 && tcp->urg != 1)
					{
					if(opt_debug)
						printf("[%s] packet is not URG, ignoring...\n", knock->name);
					continue;}
				}//<check tcp flags>
										

			/*alloc a new attempt element*/
			attempt = (attempt_t*)malloc(sizeof(attempt_t));
			if(attempt == NULL) 
				{
				printf("Error: can't alloc attempt structure\n");
				exit(EXIT_FAILURE);
				}
			/*init the attempt element*/	
			strncpy(attempt->src, IPSrc , strlen(IPSrc)+1 );
			/*strcpy(attempt->src, IPSrc);*/
			attempt->stage = 1;
			attempt->seq_start = pkt_secs;
			attempt->knock = knock;
			attempt->ipstage = 0;
			attempt->is_in_ipstage = 0;
				
			while ( (byte = strsep(&tmp_ip, ".")) ) 
				{
				attempt->ipbyte[i]=(u_char) atoi (byte);
				i++;
				}
			attempt->sequence_crypted = (u_char*) calloc (knock->seqcount, sizeof(u_char));
			strncpy (attempt->sequence_crypted, knock->sequence, knock->seqcount);
			/*calculate encrypted sequence. If encryption fails invalidate the attempt*/
			if ( encrypt_sequence (attempt, knock) )
				attempt->stage=-1;
								
			attempts_list = list_add(attempts_list, attempt);
			if (opt_debug)
				{
				printf("[%s] %s: Stage 1\n", knock->name, attempt->src);
				printf("\tCrypted sequence : ");
				for (i=0; i<knock->seqcount; i++)
					printf("%d,",attempt->sequence_crypted[i]);
				printf("%d,%d,%d,%d\n",attempt->ipbyte[0],attempt->ipbyte[1],attempt->ipbyte[2],attempt->ipbyte[3]);
				}
					
			fprintf(LogFilePtr, "[%s] %s-%s Stage %d from ip %s\n",attempt->knock->name, pkt_date, pkt_time, attempt->stage, attempt->src );				

			}//</walk knock list>
		}//</first packet of the sequence>
	  	  	  
return ;	  	
}



/*******************************************
**  Check destination port range
********************************************/
PMList* check_port_range(u_short port)
{
	PMList *lp = portspan_list;
			
	for(; lp; lp=lp->next) 
		{
		if (port >= ((portspan_t*)lp->data)->minrange && port <= ((portspan_t*)lp->data)->maxrange)
			return lp;
		}
	
	return NULL;	
} 



/*********************************************
** Demap the destination port
**********************************************/
u_short demap(u_short port, PMList* lp)
{
	port -= ((portspan_t*)lp->data)->minrange;
	
	for(lp=lp->prev; lp; lp=lp->prev)
		port += ((portspan_t*)lp->data)->maxrange - ((portspan_t*)lp->data)->minrange;
	
	return port;
}



/*********************************************
** Decrypt the first port of a sequence
*********************************************/
int decrypt_first (u_char* port)
{
	int i;

 	i=mcrypt_generic_init( hd, Key, keysize, IV);
	if (i<0) 
		{
		mcrypt_perror(i);
		return 1;
  		}
	
	if (mdecrypt_generic (hd, port, 1))
		return 1;
	mcrypt_generic_deinit (hd);
	return 0;	
}



/**********************************************
** Encrypt a sequence+ip
***********************************************/
int encrypt_sequence (attempt_t* a, knock_t* k)
{
	int i;
	
	i=mcrypt_generic_init( hd, Key, keysize, IV);
	if (i<0) 
		{
		mcrypt_perror(i);
		return 1;
  		}  		  		  	
  		
  	/* encrypt sequence */	
  	for (i=0; i < k->seqcount; i++)
  		{
		if (k->protocol[i] != IPPROTO_ICMP) 		
			if(mcrypt_generic (hd, &a->sequence_crypted[i], 1))
				return 1;
		} 
	     
   /* encrypt ip */ 
 	for (i=0; i < 4; i++)
		mcrypt_generic (hd, &a->ipbyte[i], 1);
	
	mcrypt_generic_deinit (hd);
	return 0;
} 
			 


/*****************************************
*** Parse command from conf. file
*****************************************/
char* parse_commad(char* command, char* src)
{
	char *ptr1, *ptr2;
	char origCmd[_POSIX2_LINE_MAX];
	char parsedCmd[_POSIX2_LINE_MAX] = "";

	strncpy(origCmd, command, sizeof(origCmd));
	ptr1 = origCmd;
	while((ptr2 = strstr(ptr1, "$IP"))) 
		{
		ptr2[0] = '\0';
		strcat(parsedCmd, ptr1);
		strcat(parsedCmd, src);
		ptr1 = ptr2 + 3;
		}
	strcat(parsedCmd, ptr1);
	command = parsedCmd;

	return command;
}
	
	
	
/***********************************
*** execute the parsed command
***********************************/
void exec_command(char* command, char* name, char* date, char* time)
{
	int ret;
	ret = system(command);
	if(ret == -1) 
		{
		if(opt_verbose)
			printf("** Error **: command fork failed! for command %s of sequence %s\n",command, name);	
		fprintf(LogFilePtr, "[%s][ERROR] %s-%s command fork failed for %s\n", name, date, time, command );		
		} 
	else if(ret != 0) 
		{
		if(opt_verbose)		
			printf("** Warning **: command %s of sequence %s returned non-zero status code (%d)\n", command, name, ret);
		fprintf(LogFilePtr, "[%s][WARNING] %s-%s : command %s returned non-sero status code (%d)\n", name, date, time, command, ret);
		}
	else if(!ret)
		{
		fprintf(LogFilePtr, "[%s] %s-%s command : %s\t[OK]\n", name, date, time, command );				
		if(opt_verbose)
			{
			printf("[%s] command %s", name, command);		
			printf("\t[%sOK%s]\n", GREEN, DEFAULT);
			}
		}

	return;
}	
