/*
 * servknock.h
 *
 *  copyright (c) 2004 by Beretta Andrea <beretta.andrea@libero.it>
 *  and Bianchi Riccardo <ikir@iol.it>
 *
 *  Thanks to Judd Vinet <jvinet@zeroflux.org>
 *  and Paolo Carpo <snifth@spine-group.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 */

/*includes*/ 
#include <pcap.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
/*waitpid*/
#include <sys/wait.h>
/*libmcrypt encryption/decryption library*/
#include <mcrypt.h>
/*hashing*/
#include <mhash.h>
/*operations on list*/
#include "list.h"
/*misc ops*/
#include "misc.h"
/*little/big endian definitions*/
#include "bytesex.h"


/*max bytes to capture*/
#define MAX_BYTES 144
/*max knock sequence length*/
#define SEQ_MAX 15
/*default knock sequence timeout*/
#define SEQ_TIMEOUT 25
/*default command timeout*/
#define CMD_TIMEOUT 15
/* ANSI escape sequences*/
#define WHITE	"\033[1;37m"
#define GREEN	"\033[0;32m"
#define RED		"\033[1;31m"
#define DEFAULT	"\033[0m"
/* header protocol lenghts */
#define IPV4_H		0x14	/**< IPv4 header: 20 bytes */



/* knock tuples struct */
typedef struct knock_struct {
	char name[128];
	u_short seqcount;
	u_char sequence[SEQ_MAX];
	u_short protocol[SEQ_MAX];
	time_t seq_timeout;
	PMList *start_command_list;
	time_t cmd_timeout;
	PMList *stop_command_list;
	u_short flag_fin;
	u_short flag_syn;
	u_short flag_rst;
	u_short flag_psh;
	u_short flag_ack;
	u_short flag_urg;
} knock_t;


/* knock attempt struct*/
typedef struct attempt {
	knock_t *knock;
	short stage;
	char src[16];
	time_t seq_start;
	u_char ipbyte[4];
	short ipstage;
	int is_in_ipstage;
	u_char* sequence_crypted;
} attempt_t;


/*command struct*/
typedef struct command {
	char *bin;
	} command_t;


/* port span struct*/
typedef struct portspan {
	u_int minrange;
	u_int maxrange;
} portspan_t;


/* IPv4 header, version 4, Static header size: 20 bytes */
struct ipv4_hdr
{
#if defined(BYTE_ORDER_LITTLE_ENDIAN)
	u_int8_t ip_hl:4,	/* header length */
	ip_v:4;				/* version */
#elif defined (BYTE_ORDER_BIG_ENDIAN)	
	u_int8_t ip_v:4,	/* header length */
	ip_hl:4;			/* version */
#else
	#error  "Please, edit Makefile and add -DBYTE_ORDER_(BIG|LITTLE)_ENDIAN"		
#endif
	u_int8_t ip_tos;	/* type of service */
	u_int16_t ip_len;	/* total length */
	u_int16_t ip_id;	/* identification */
	u_int16_t ip_off;

	u_int8_t ip_ttl;	/* time to live */
	u_int8_t ip_p;		/* protocol */
	u_int16_t ip_sum;	/* checksum */
	struct in_addr ip_src, ip_dst;	/* source and dest address */
};

/* UDP header */
struct udp_hdr
{
	u_int16_t uh_sport;		/* soure port */
	u_int16_t uh_dport;		/* destination port */
	u_int16_t uh_ulen;		/* length */
	u_int16_t uh_sum;		/* checksum */
};


/* TCP header (linux/tcp.h) */
struct tcp_hdr {
	u_int16_t	th_sport;	/* source port */
	u_int16_t	th_dport;	/* destination port */
	u_int32_t	th_seq;		/* sequence number */
	u_int32_t	th_ack;		/* acknowledgement number */
#if defined (BYTE_ORDER_LITTLE_ENDIAN)
	u_int8_t	th_x2:4,	/* (unused) */
				th_off:4;	/* data offset */
#elif defined (BYTE_ORDER_BIG_ENDIAN)
	u_int8_t	th_off:4,	/* data offset */
				th_x2:4;	/* (unused) */
#else
	#error  "Please, edit Makefile and add -DBYTE_ORDER_(BIG|LITTLE)_ENDIAN"				
#endif		
					
	/*flags*/
	u_int8_t	fin:1,
				syn:1,
				rst:1,
				psh:1,
				ack:1,
				urg:1;
	
	u_int16_t   th_win;		/* window */
	u_int16_t   th_sum;		/* checksum */
	u_int16_t   th_urp;		/* urgent pointer */
};
	    
	    

/*global vars*/
knock_t *knock=NULL;
attempt_t* attempt=NULL;
command_t *command=NULL;
portspan_t *port_range=NULL;
/*list of various knock*/
PMList *knocks_list = NULL;
/*list of various attempts, 1 for ip*/
PMList *attempts_list=NULL;
/*list of ranges in portspan*/
PMList *portspan_list=NULL;
/*pcap file descriptor */
pcap_t* descr;
/* Mcrypt encryption descriptor*/
MCRYPT hd;		
/*server ip address */
char IfaceIp[32]; 		
/*sniffed interface*/
char IfaceName[20];
/*cypher*/
char CryptAlgName[100];
char temp_iv[16];
u_char *IV=NULL;
/*password used to generate the key*/
char password[16];
/*key used for encryption/decryption*/
u_char *Key;
/*key size*/	
int keysize;
/*pointer to logfile*/
FILE *LogFilePtr;	
/*debug option*/
int opt_debug=0;
/*verbose option*/
int opt_verbose=0;


/* prototypes of functions */
int main(int argc, char **argv);
/* signal handler */
void sigexit();
/* handler for SIGCHLD */
void sigzombie(void);
/* Print program usage */
void give_help(void);
/* Parse config file */
void parse_config (char *filename);
/* called with every packet, process it*/
void proc_packet(char *packet, int offset, struct pcap_pkthdr* hdr);
/*check destination port range*/
PMList* check_port_range(u_short port);
/*demap destination port*/
u_short demap(u_short port, PMList* lp);
/*decrypt the first port of a sequence*/
int decrypt_first (u_char* port);
/*encrypt a sequence+ip*/
int encrypt_sequence (attempt_t* a, knock_t* k);
/*parse command from conf_file*/
char* parse_commad(char* command, char* src);
/*execute the parsed command*/
void exec_command(char* command, char* name, char* date, char* time);
